"""This module is for the translation mapping data of the
following document:

Document file: fps.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# FPS enum':
    '',

    'This page explains the `FPS` enum class.':
    '',

    '## What class is this?':
    '',

    'The `FPS` enum class is the definition of each FPS (frames per second). The timer uses this enum to determine the timer tick interval.':  # noqa
    '',

    '## Basic usage':
    '',

    'There is an enum definition of FPS in 5 intervals. The `Timer` class `delay` argument is acceptable `FPS` enum value. For example, specify the `FPS.FPS_60` value to that argument. A timer interval becomes approximately `16.6666667` milliseconds. Similarly, it becomes the `33.3333333` milliseconds when you specify the `FPS.FPS_30` value.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.rotation_around_center += 1\n\n\nap.Stage(\n    stage_width=350, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\noptions: _RectOptions = {\'rectangle\': rectangle_1}\ntimer_1: ap.Timer = ap.Timer(\n    handler=on_timer, delay=ap.FPS.FPS_10, options=options)\ntimer_1.start()\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\noptions = {\'rectangle\': rectangle_2}\ntimer_2: ap.Timer = ap.Timer(\n    handler=on_timer, delay=ap.FPS.FPS_30, options=options)\ntimer_2.start()\n\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=250, y=50, width=50, height=50)\noptions = {\'rectangle\': rectangle_3}\ntimer_3: ap.Timer = ap.Timer(\n    handler=on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer_3.start()\n\nap.save_overall_html(\n    dest_dir_path=\'fps_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/fps_basic_usage/index.html" width="350" height="150"></iframe>':  # noqa
    '',

    '## See also':
    '',

    '- [Timer class delay setting](timer_delay.md)':
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: dblclick.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Double click interface':
    '',

    'This page explains the `dblclick` (double-click) interface.':
    '',

    '## What interface is this?':
    '',

    'The `dblclick` interface binds the double-click event to any `DisplayObject` instance (e.g., `Sprite`\\, `Rectangle`\\, and so on). If you double-click on that instance, this interface calls the registered handler function.':  # noqa
    '',

    '## See also':
    '',

    'The following page describes the basic mouse event interfaces.\n\n- [Basic mouse event interfaces](mouse_event_basic.md)':  # noqa
    '',

    '## Basic usage of the dblclick interface':
    '',

    'Each `DisplayObject` instance has the `dblclick` method, and you can bind handlers by that.\n\nThe following example binds the double-click event handler to the rectangle. If you double-click on that instance, the rectangle color changes from cyan to magenta.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when double-clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String(\'#f0a\')\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.dblclick(on_double_click)\n\nap.save_overall_html(\n    dest_dir_path=\'./dblclick_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/dblclick_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Basic usage of the unbind_dblclick interfaces':
    '',

    'The `unbind_dblclick` interface can remove the single binded double-click event from a `DisplayObject` instance. The `unbind_dblclick_all` interface removes all double-click events.\n\nThe following example removes the double click event by the `unbind_dblclick` method. If you double-click the rectangle, nothing happens.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when double-clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String(\'#f0a\')\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.dblclick(on_double_click)\nrectangle.unbind_dblclick(on_double_click)\n\nap.save_overall_html(\n    dest_dir_path=\'./unbind_dblclick_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/unbind_dblclick_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## dblclick API':
    '',

    '<!-- Docstring: apysc._event.double_click_interface.DoubleClickInterface.dblclick -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `dblclick(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType], *, options:Union[~_O, NoneType]=None) -> str`<hr>\n\n**[Interface summary]** Add a double-click event listener setting.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Callable that would be called when double-clicking this instance.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to be passed to a handler.\n\n<hr>\n\n**[Returns]**\n\n- `name`: str\n  - Handler\'s name.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_double_click(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.dblclick(on_double_click)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [About the handler options’ type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## unbind_dblclick API':
    '',

    '<!-- Docstring: apysc._event.double_click_interface.DoubleClickInterface.unbind_dblclick -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_dblclick(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType]) -> None`<hr>\n\n**[Interface summary]** Unbind a specified handler\'s double click event.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Unbinding target Callable.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_double_click(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_dblclick(on_double_click)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.dblclick(on_double_click)\n```':  # noqa
    '',

    '':
    '',

    '## unbind_dblclick_all API':
    '',

    '<!-- Docstring: apysc._event.double_click_interface.DoubleClickInterface.unbind_dblclick_all -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_dblclick_all(self) -> None`<hr>\n\n**[Interface summary]** Unbind all double click events.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_double_click(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_dblclick_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.dblclick(on_double_click)\n```':  # noqa
    '',

}



"""Class implementation for the attribute linking interface.

This interface updates an old property value to achieve
consistency in the handler functions.
"""

from typing import Dict
from typing import List
from typing import Union

from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String

_AttrName = str
_Attr = Union[Int, Number, String, Boolean]


class AttrLinkingInterface:

    _attr_linking_stack: Dict[_AttrName, List[_Attr]]

    def _initialize_attr_linking_stack(
            self, *, attr_name: str) -> None:
        """
        Initialize the _attr_linking_stack attribute if
        this instance does not initialize it yet.

        Parameters
        ----------
        attr_name : str
            Target attribute name.
        """
        if not hasattr(self, '_attr_linking_stack'):
            self._attr_linking_stack = {}
        if attr_name in self._attr_linking_stack:
            return
        self._attr_linking_stack[attr_name] = []

    def _append_attr_to_linking_stack(
            self, *, attr: _Attr, attr_name: str) -> None:
        """
        Append an attribute to the linking attribute stack.

        Parameters
        ----------
        attr : Int or Number or String or Boolean
            Target attribute to be appended.
        attr_name : str
            Target attribute name.
        """
        self._initialize_attr_linking_stack(attr_name=attr_name)
        if self._is_target_attr_already_linked(
                attr=attr, attr_name=attr_name):
            return
        self._attr_linking_stack[attr_name].append(attr)

    def _is_target_attr_already_linked(
            self, *, attr: _Attr, attr_name: str) -> bool:
        """
        Get a boolean value whether this instance already
        appends a specified attribute to the linking
        attribute stack.

        Parameters
        ----------
        attr : Int or Number or String or Boolean
            Target attribute to be appended.
        attr_name : str
            Target attribute name.

        Returns
        -------
        result : bool
            If this instance already appends a specified
            attribute to the linking stack, this interface
            returns True.
        """
        self._initialize_attr_linking_stack(attr_name=attr_name)
        for in_stack_value in self._attr_linking_stack[attr_name]:
            if in_stack_value.variable_name == attr.variable_name:
                return True
        return False

    def _append_applying_new_attr_val_exp(
            self, *, new_attr: _Attr, attr_name: str) -> None:
        """
        Append the expression of applying a new attribute
        value to each stacked value.

        Parameters
        ----------
        new_attr : Int or Number or String or Boolean
            New attribute value.

        attr_name : str
            Target attribute name.
        """
        import apysc as ap
        self._initialize_attr_linking_stack(attr_name=attr_name)
        if not self._attr_linking_stack:
            return
        new_attr_name: str = new_attr.variable_name
        expression: str = ''
        for stacked_value in self._attr_linking_stack[attr_name]:
            if stacked_value.variable_name == new_attr_name:
                continue
            if expression != '':
                expression += '\n'
            expression += (
                f'{stacked_value.variable_name} = {new_attr_name};'
            )
        ap.append_js_expression(expression=expression)



"""Class implementation for width interface.
"""

from typing import Dict
from typing import Union

from apysc._animation.animation_width_interface import AnimationWidthInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_interface import AttrLinkingInterface
from apysc._type.int import Int
from apysc._type.revert_interface import RevertInterface


class WidthInterface(
        AnimationWidthInterface, RevertInterface, AttrLinkingInterface):

    _width: Int

    def _initialize_width_if_not_initialized(self) -> None:
        """
        Initialize _width attribute if this instance does not
        initialize it yet.
        """
        if hasattr(self, '_width'):
            return
        self._width = Int(0)

        self._append_width_attr_linking_setting()

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='WidthInterface')
    def _append_width_attr_linking_setting(self) -> None:
        """
        Append a width attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._width, attr_name='width')
        self._append_attr_to_linking_stack(
            attr=self._width, attr_name='width')

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='WidthInterface')
    def width(self) -> Int:
        """
        Get this instance's width.

        Returns
        -------
        width : Int
            This instance's width.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> rectangle.width = ap.Int(100)
        >>> rectangle.width
        Int(100)
        """
        import apysc as ap
        from apysc._type import value_util
        self._initialize_width_if_not_initialized()
        width: ap.Int = value_util.get_copy(value=self._width)
        return width

    @width.setter
    def width(self, value: Int) -> None:
        """
        Update this instance's width.

        Parameters
        ----------
        value : Int
            Width value to set.
        """
        from apysc._html.debug_mode import DebugInfo
        with DebugInfo(
                callable_='width', args=[value], kwargs={},
                module_name=__name__,
                class_name=WidthInterface.__name__):
            self._update_width_and_skip_appending_exp(value=value)
            self._width._append_incremental_calc_substitution_expression()
            self._append_width_update_expression()

            self._append_width_attr_linking_setting()

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='WidthInterface')
    def _append_width_update_expression(self) -> None:
        """
        Append width updating expression.
        """
        import apysc as ap
        from apysc._type import value_util
        width_str: str = value_util.get_value_str_for_expression(
            value=self._width)
        expression: str = (
            f'{self.variable_name}.width({width_str});'
        )
        ap.append_js_expression(expression=expression)

    def _update_width_and_skip_appending_exp(
            self, *, value: Union[int, Int]) -> None:
        """
        Update width value and skip appending expression.

        Parameters
        ----------
        value : int or Int
            Width value to set.
        """
        import apysc as ap
        from apysc._converter import cast
        from apysc._validation import size_validation
        self._initialize_width_if_not_initialized()
        value = cast.to_int_from_float(int_or_float=value)
        size_validation.validate_size_is_int(size=value)
        size_validation.validate_size_is_gte_zero(size=value)
        if isinstance(value, ap.Int):
            value_: ap.Int = value
        else:
            value_ = Int(value)
        self._width = value_

    _width_snapshots: Dict[str, int]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_width_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name='_width_snapshots',
            value=int(self._width._value), snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._width._value = self._width_snapshots[snapshot_name]



"""Base class implementation for the path data.
"""

from abc import ABC
from abc import abstractmethod
from typing import Union

from apysc._branch._else import Else
from apysc._branch._if import If
from apysc._geom.path_label import PathLabel
from apysc._geom.relative_interface import RelativeInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.string import String


class PathDataBase(RelativeInterface, ABC):
    """
    Base class for the path data.
    """

    _path_label: PathLabel

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathDataBase')
    def __init__(
            self, *, path_label: PathLabel,
            relative: Union[bool, Boolean]) -> None:
        """
        Base class for the path data.

        Parameters
        ----------
        path_label : PathLabel
            Target (SVG's) path label.
        relative : bool or Boolean
            A boolean value indicates whether the path
            coordinates are relative or not (absolute).
        """
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_boolean_from_builtin_val
        self._path_label = path_label
        self.relative = get_copied_boolean_from_builtin_val(
            bool_val=relative)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathDataBase')
    def _get_svg_char(self) -> String:
        """
        Get a SVG character (e.g., 'M' or 'm') from the
        current setting.

        Returns
        -------
        svg_char : String
            Target SVG character.
        """
        svg_char_: str = self._path_label.value
        if self._relative._value:
            svg_char: String = String(svg_char_.lower())
        else:
            svg_char = String(svg_char_)
        with If(self._relative, locals_=locals()):
            svg_char.value = svg_char_.lower()
        with Else(locals_=locals()):
            svg_char.value = svg_char_
        return svg_char

    @abstractmethod
    def _get_svg_str(self) -> str:
        """
        Get a path's SVG string created with the current setting.
        """



"""Class implementation for the animation event.
"""

from typing import Generic
from typing import TypeVar

from apysc._animation import animation_base
from apysc._event.event import Event
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_interface import VariableNameInterface

_T = TypeVar('_T', bound=VariableNameInterface)


class AnimationEvent(Event, Generic[_T]):
    """
    Animation event class.

    Examples
    --------
    >>> import apysc as ap
    >>> def on_animation_complete(
    ...         e: ap.AnimationEvent[ap.Rectangle],
    ...         options: dict) -> None:
    ...     rectangle: ap.Rectangle = e.this.target
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color='#0af')
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50)
    >>> _ = rectangle.animation_x(
    ...     x=100).animation_complete(on_animation_complete)
    """

    _this: 'animation_base.AnimationBase[_T]'

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='AnimationEvent')
    def __init__(
            self, *,
            this: 'animation_base.AnimationBase[_T]') -> None:
        """
        Animation event class.

        Parameters
        ----------
        this : AnimationBase
            Animation setting instance.

        Examples
        --------
        >>> import apysc as ap
        >>> def on_animation_complete(
        ...         e: ap.AnimationEvent[ap.Rectangle],
        ...         options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this.target
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.animation_x(
        ...     x=100).animation_complete(on_animation_complete)

        """
        from apysc._expression import var_names
        super(AnimationEvent, self).__init__(
            this=this, type_name=var_names.ANIMATION_EVENT)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='AnimationEvent')
    def this(self) -> 'animation_base.AnimationBase[_T]':
        """
        Get an animation setting instance of listening to this event.

        Returns
        -------
        this : AnimationBase
            Instance of listening to this event.

        Examples
        --------
        >>> import apysc as ap
        >>> def on_animation_complete(
        ...         e: ap.AnimationEvent[ap.Rectangle],
        ...         options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this.target
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.animation_x(
        ...     x=100).animation_complete(on_animation_complete)
        """
        return self._this



"""This module is for the translation mapping data of the
following document:

Document file: mousemove.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Mousemove interface':
    '',

    'This page explains the `mousemove` interface.':
    '',

    '## What interface is this?':
    '',

    'The `mousemove` interface binds the mouse moving event handler to any `DisplayObject` instance. If you move the mouse cursor on that instance, the interface calls the registered handler.':  # noqa
    '',

    '## See also':
    '',

    'The following page describes the basic mouse event interfaces.\n\n- [Basic mouse event interfaces](mouse_event_basic.md)':  # noqa
    '',

    '## Basic usage':
    '',

    'Each `DisplayObject` instance has the `mousemove` method, and you can bind handlers by that.\n\nThe following example binds the mouse move event handler to the circle. So if you move a mouse cursor on that, the circle follows the cursor position.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mousemove(\n        e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.x = e.stage_x\n    circle.y = e.stage_y\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\ncircle.mousemove(on_mousemove)\n\nap.save_overall_html(\n    dest_dir_path=\'mousemove_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/mousemove_basic_usage/index.html" width="200" height="200"></iframe>':  # noqa
    '',

    '## Unbind interfaces':
    '',

    '`unbind_mousemove` interface can remove the binding of the mouse move event from the `DisplayObject`\\.\n\nIn the following example, the interface removes the mouse move event handler if you click the circle.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mousemove(\n        e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.x = e.stage_x\n    circle.y = e.stage_y\n\n\ndef on_click(e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.unbind_mousemove(handler=on_mousemove)\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\ncircle.mousemove(on_mousemove)\ncircle.click(on_click)\n\nap.save_overall_html(\n    dest_dir_path=\'mousemove_unbind_interface/\')\n```':  # noqa
    '',

    '<iframe src="static/mousemove_unbind_interface/index.html" width="200" height="200"></iframe>\n\nThere are also existing the `unbind_mousemove_all` interface. This interface unbinds all the handlers from the target `DisplayObject` instance.':  # noqa
    '',

    '## mousemove API':
    '',

    '<!-- Docstring: apysc._event.mouse_move_interface.MouseMoveInterface.mousemove -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `mousemove(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType], *, options:Union[~_O, NoneType]=None) -> str`<hr>\n\n**[Interface summary]** Add mouse move event listener setting.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Callable that would be called when mousemove on this instance.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to be passed to a handler.\n\n<hr>\n\n**[Returns]**\n\n- `name`: str\n  - Handler\'s name.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousemove(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace(\'stage_x:\', stage_x)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousemove(on_mousemove)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [About the handler options’ type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## unbind_mousemove API':
    '',

    '<!-- Docstring: apysc._event.mouse_move_interface.MouseMoveInterface.unbind_mousemove -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mousemove(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType]) -> None`<hr>\n\n**[Interface summary]** Unbind a specified handler\'s mouse move event.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Unbinding target Callable.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousemove(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace(\'stage_x:\', stage_x)\n>>> def on_click(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.unbind_mousemove(on_mousemove)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousemove(on_mousemove)\n>>> _ = rectangle.click(on_click)\n```':  # noqa
    '',

    '':
    '',

    '## unbind_mousemove_all API':
    '',

    '<!-- Docstring: apysc._event.mouse_move_interface.MouseMoveInterface.unbind_mousemove_all -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mousemove_all(self) -> None`<hr>\n\n**[Interface summary]** Unbind all mouse move events.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousemove(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace(\'stage_x:\', stage_x)\n>>> def on_click(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.unbind_mousemove_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousemove(on_mousemove)\n>>> _ = rectangle.click(on_click)\n```':  # noqa
    '',

}



"""Class implementation for the `visible` interface.
"""

from typing import Dict

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_interface import AttrLinkingInterface
from apysc._type.boolean import Boolean
from apysc._type.revert_interface import RevertInterface
from apysc._type.variable_name_interface import VariableNameInterface


class VisibleInterface(
        VariableNameInterface, RevertInterface, AttrLinkingInterface):

    _visible: Boolean

    def _initialize_visible_if_not_initialized(self) -> None:
        """
        Initialize _visible attribute if this instance does not
        initialize it yet.
        """
        if hasattr(self, '_visible'):
            return
        self._visible = Boolean(True)

        self._append_visible_attr_linking_setting()

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='VisibleInterface')
    def _append_visible_attr_linking_setting(self) -> None:
        """
        Append a visible attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._visible, attr_name='visible')
        self._append_attr_to_linking_stack(
            attr=self._visible, attr_name='visible')

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='VisibleInterface')
    def visible(self) -> Boolean:
        """
        Get a visibility value of this instance.

        Returns
        -------
        result : Boolean
            If this instance is visible, this interface returns True.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> rectangle.visible = ap.Boolean(False)
        >>> rectangle.visible
        Boolean(False)
        """
        from apysc._type import value_util
        self._initialize_visible_if_not_initialized()
        return value_util.get_copy(value=self._visible)

    @visible.setter
    def visible(self, value: Boolean) -> None:
        """
        Update a visibility value of this instance.

        Parameters
        ----------
        value : Boolean
            Boolean value to set.
        """
        from apysc._html.debug_mode import DebugInfo
        with DebugInfo(
                callable_='visible', args=[value], kwargs={},
                module_name=__name__,
                class_name=VisibleInterface.__name__):
            import apysc as ap
            from apysc._validation import bool_validation
            bool_validation.validate_bool(value=value)
            if isinstance(value, bool):
                value = ap.Boolean(value)
            self._visible = value
            self._append_visible_update_expression()

            self._append_visible_attr_linking_setting()

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='VisibleInterface')
    def _append_visible_update_expression(self) -> None:
        """
        Append visible property updating expression.
        """
        import apysc as ap
        expression: str = (
            f'if ({self._visible.variable_name}) {{'
            f'\n  {self.variable_name}.show();'
            '\n}else {'
            f'\n  {self.variable_name}.hide();'
            '\n}'
        )
        ap.append_js_expression(expression=expression)

    _visible_snapshots: Dict[str, bool]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_visible_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name='_visible_snapshots',
            value=self._visible._value, snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value is snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._visible._value = self._visible_snapshots[snapshot_name]



"""This module is for the document splitting utility.
"""


from typing import List
from typing import Union


class Heading:
    """This class is for the document's heading.
    """

    _text: str
    _overall_text: str
    _sharp_num: int

    def __init__(self, *, heading_text: str) -> None:
        """
        This class is for the document's heading.

        Parameters
        ----------
        heading_text : str
            A heading text of a document.
        """
        self._overall_text = heading_text
        sharp_num: int = 0
        for char in heading_text:
            if char != '#':
                break
            sharp_num += 1
        text: str = heading_text.replace('#', '', sharp_num).strip()
        self._text = text
        self._sharp_num = sharp_num

    @property
    def text(self) -> str:
        """
        Get a heading text except first sharp symbols.

        Returns
        -------
        text : str
            A heading text except first sharp symbols.
        """
        return self._text

    @property
    def overall_text(self) -> str:
        """
        Get an overall heading text.

        Returns
        -------
        overall_text : str
            A overall heading text.
        """
        return self._overall_text

    @property
    def sharp_num(self) -> int:
        """
        Get a sharp number of a heading text's sharp symbols.

        Returns
        -------
        sharp_num : int
            A sharp number of a heading text's sharp symbols.
        """
        return self._sharp_num


class BodyText:
    """This class is for a document body text.
    """

    _text: str

    def __init__(self, *, text: str) -> None:
        """
        This class is for a document body text.

        Parameters
        ----------
        text : str
            A document body text.
        """
        self._text = text.strip()

    @property
    def text(self) -> str:
        """
        Get a document body text.

        Returns
        -------
        text : str
            A document body text.
        """
        return self._text


class CodeBlock:

    _code_block: str
    _overall_code_block: str
    _code_type: str = ''

    def __init__(self, *, code_block: str) -> None:
        """
        The class for a document code block.

        Parameters
        ----------
        code_block : str
            A document code block.
        """
        code_block = code_block.strip()
        self._overall_code_block = code_block
        code_block_lines: List[str] = []
        lines: List[str] = code_block.splitlines()
        for line in lines:
            if line.startswith('```') and self._code_type == '':
                self._code_type = line.replace('```', '').strip()
                continue
            if line.startswith('```'):
                break
            if line == '# runnable':
                continue
            code_block_lines.append(line)
        self._code_block = '\n'.join(code_block_lines).strip()

    @property
    def code_block(self) -> str:
        """
        Get a code block string except triple-quotations.

        Returns
        -------
        code_block : str
            A code block string except triple-quotations.
        """
        return self._code_block

    @property
    def overall_code_block(self) -> str:
        """
        Get an overall code block string.

        Returns
        -------
        overall_code_block : str
            An overall code block string.
        """
        return self._overall_code_block

    @property
    def code_type(self) -> str:
        """
        Get a code type of a code block.

        Returns
        -------
        code_type : str
            A code type (e.g., 'py').
        """
        return self._code_type


def split_markdown_document(
        *,
        markdown_txt: str) -> List[Union[Heading, BodyText, CodeBlock]]:
    """
    Split a specified markdown document to `Heading`,
    `BodyText`, and `CodeBlock` values.

    Parameters
    ----------
    markdown_txt : str
        A target markdown document.

    Returns
    -------
    splitted_values : list of Heading, BodyText, and CodeBlock
        A list of splitted `Heading`, `BodyText`, and `CodeBlock`
        values.
    """
    is_code_block: bool = False
    current_code_block_lines: List[str] = []
    current_body_text_lines: List[str] = []
    lines: List[str] = markdown_txt.splitlines()
    splitted_values: List[Union[Heading, BodyText, CodeBlock]] = []
    for line in lines:
        if not is_code_block and line.startswith('```'):
            _create_body_text_and_append_to_list_if_values_exist(
                splitted_values=splitted_values,
                body_text_lines=current_body_text_lines)
            is_code_block = True
            current_code_block_lines.append(line)
            continue

        if is_code_block:
            current_code_block_lines.append(line)
            if line.startswith('```'):
                code_block: CodeBlock = _create_code_block_from_list(
                    code_block_lines=current_code_block_lines)
                splitted_values.append(code_block)
                is_code_block = False
                continue
            continue

        if line.startswith('#'):
            _create_body_text_and_append_to_list_if_values_exist(
                splitted_values=splitted_values,
                body_text_lines=current_body_text_lines)
            splitted_values.append(Heading(heading_text=line))
            continue

        current_body_text_lines.append(line)

    _create_body_text_and_append_to_list_if_values_exist(
        splitted_values=splitted_values,
        body_text_lines=current_body_text_lines)
    return splitted_values


def _create_body_text_and_append_to_list_if_values_exist(
        *,
        splitted_values: List[Union[Heading, BodyText, CodeBlock]],
        body_text_lines: List[str]) -> None:
    """
    Create a body text instance from a specified body
    text lines list and append it to a result list.
    This interface skips the appending if a specified
    body text line list is blank.

    Notes
    -----
    This function clears a specified body text lines list
    at the end.

    Parameters
    ----------
    splitted_values : list of Heading, BodyText, and CodeBlock
        A result list to append a body text instance.
    body_text_lines : list of str
        A target body text line's list.
    """
    if not body_text_lines:
        return
    body_text_str: str = '\n'.join(body_text_lines)
    splitted_values.append(BodyText(text=body_text_str))
    body_text_lines.clear()


def _create_code_block_from_list(
        *, code_block_lines: List[str]) -> CodeBlock:
    """
    Create a code block instance from a specified lines list.

    Notes
    -----
    This function clears a specified list at the end of the
    function.

    Parameters
    ----------
    code_block_lines : list of str
        A target code block lines.

    Returns
    -------
    code_block : CodeBlock
        A created code block instance.
    """
    code_block_str: str = '\n'.join(code_block_lines)
    code_block: CodeBlock = CodeBlock(code_block=code_block_str)
    code_block_lines.clear()
    return code_block



"""This module is for the translation mapping data of the
following document:

Document file: timer_event.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# TimerEvent class':
    '',

    'This page explains the `TimerEvent` class.':
    '',

    '## What class is this?':
    '',

    'The `TimerEvent` class is the event class that a timer passes to a timer event handler function, such as the `Timer` class constructor or the `timer_complete` function\\.':  # noqa
    '',

    '## Basic usage':
    '',

    'Each timer event handler\'s `e` argument becomes the `TimerEvent` class instance.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.rotation_around_center += 1\n\n\ndef on_timer_complete(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The handler that the timer calls when completed.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(\'Timer complete!\')\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\noptions: _RectOptions = {\'rectangle\': rectangle}\ntimer: ap.Timer = ap.Timer(\n    handler=on_timer, delay=33.3, options=options)\ntimer.start()\ntimer.timer_complete(handler=on_timer_complete)\n\nap.save_overall_html(\n    dest_dir_path=\'timer_event_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/timer_event_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## This attribute':
    '',

    'The `TimerEvent` instance\'s `this` attribute becomes the target `Timer` instance, and you can use each timer instance interface from it.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.rotation_around_center += 1\n    ap.trace(\'Current timer count: \', e.this.current_count)\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\noptions: _RectOptions = {\'rectangle\': rectangle}\ntimer: ap.Timer = ap.Timer(\n    handler=on_timer, delay=16.6, options=options)\ntimer.start()\n\nap.save_overall_html(\n    dest_dir_path=\'timer_event_this_attribute/\')\n```':  # noqa
    '',

    '<iframe src="static/timer_event_this_attribute/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## TimerEvent constructor API':
    '',

    '<!-- Docstring: apysc._event.timer_event.TimerEvent.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, this:\'timer.Timer\') -> None`<hr>\n\n**[Interface summary]** Timer event class.<hr>\n\n**[Parameters]**\n\n- `this`: Timer\n  - Target timer instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options[\'rectangle\']\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> options: RectOptions = {\'rectangle\': rectangle}\n>>> ap.Timer(\n...     on_timer, delay=ap.FPS.FPS_60, options=options,\n... ).start()\n```':  # noqa
    '',

    '':
    '',

    '## this attribute API':
    '',

    '<!-- Docstring: apysc._event.timer_event.TimerEvent.this -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a timer instance of listening to this event.<hr>\n\n**[Returns]**\n\n- `this`: TImer\n  - Instance of listening to this event.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options[\'rectangle\']\n...     rectangle.x += 1\n...     with ap.If(rectangle.x >= 100):\n...         timer: ap.Timer = e.this\n...         timer.stop()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> options: RectOptions = {\'rectangle\': rectangle}\n>>> ap.Timer(\n...     on_timer, delay=ap.FPS.FPS_60, options=options,\n... ).start()\n```':  # noqa
    '',

}



"""This module is for the definitions of the mouse
event types.
"""

from enum import Enum


class MouseEventType(Enum):
    """
    Mouse event type definitions.
    """

    CLICK = 'click'
    DBLCLICK = 'dblclick'
    MOUSEDOWN = 'mousedown'
    MOUSEUP = 'mouseup'
    MOUSEOVER = 'mouseover'
    MOUSEOUT = 'mouseout'
    MOUSEMOVE = 'mousemove'



"""This module is for the translation mapping data of the
following document:

Document file: timer_start_and_stop.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Timer class start and stop interfaces':
    '',

    'This page explains the interface of the Timer class `start` and `stop` methods.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `start` method interface starts a timer. Conversely, the `stop` method interface stops a timer.':  # noqa
    '',

    '## Basic usage':
    '',

    'Each `start` and `stop` method has no arguments. The following example starts the timer when you click the rectangle and stops when the count reaches 100.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    options_: _RectOptions = {\'rectangle\': e.this}\n    timer: ap.Timer = ap.Timer(\n        handler=on_timer, delay=16, repeat_count=100,\n        options=options_)\n    timer.start()\n    e.this.unbind_click(handler=on_rectangle_click)\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler what a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.x += 1\n    timer: ap.Timer = e.this\n    condition: ap.Boolean = timer.current_count == 100\n    with ap.If(condition):\n        timer.stop()\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(\n    dest_dir_path=\'timer_start_and_stop_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/timer_start_and_stop_basic_usage/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## start API':
    '',

    '<!-- Docstring: apysc._time.timer.Timer.start -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `start(self) -> None`<hr>\n\n**[Interface summary]** Start this timer.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> _ = ap.Timer(\n...     on_timer, delay=33.3, repeat_count=50).start()\n```':  # noqa
    '',

    '':
    '',

    '## stop API':
    '',

    '<!-- Docstring: apysc._time.timer.Timer.stop -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `stop(self) -> None`<hr>\n\n**[Interface summary]** Stop this timer.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options[\'rectangle\']\n...     rectangle.x += 1\n...     with ap.If(rectangle.x > 100):\n...         timer: ap.Timer = e.this\n...         timer.stop()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> options: RectOptions = {\'rectangle\': rectangle}\n>>> _ = ap.Timer(\n...     on_timer, delay=33.3, options=options).start()\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: point2d.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Point2D':
    '',

    'This page explains the `Point2D` class.':
    '',

    '## What is the Point2D class?':
    '',

    'The `Point2D` class is the 2D coordinates class interface. This interface handles the x-coordinate and y-coordinate. This interface is used, for example, the `Polygon` class drawing to specify each vertex point.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `Point2D` class constructor requires the `x` and `y` arguments. Both parameters type is the Python built-in `int` or `Int`\\.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\npoint_1: ap.Point2D = ap.Point2D(x=10, y=20)\n\nx: ap.Int = ap.Int(10)\ny: ap.Int = ap.Int(20)\npoint_2: ap.Point2D = ap.Point2D(x=x, y=y)\n```':  # noqa
    '',

    '':
    '',

    '## X and y getter interfaces':
    '',

    'The `Point2D` class `x` and `y` property interfaces returns the `Int` type value, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\npoint: ap.Point2D = ap.Point2D(x=10, y=20)\nassert point.x == 10\nassert point.y == 20\n```':  # noqa
    '',

    '':
    '',

    '## X and y setter interfaces':
    '',

    'The `x` and `y` property can be updated with an `Int` type value, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\npoint: ap.Point2D = ap.Point2D(x=10, y=20)\npoint.x = ap.Int(30)\nassert point.x == 30\n```':  # noqa
    '',

    '':
    '',

    '## Usage example of the draw_polygon interface':
    '',

    'The `draw_polygon` interface requires the `Point2D` list argument so that this section shows the example of the `Point2D` class with that drawing interface.\n\nThe following draws the triangle vector graphics by specifying the three points:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nsprite.graphics.draw_polygon(\n    points=[\n        ap.Point2D(x=75, y=50),\n        ap.Point2D(x=50, y=100),\n        ap.Point2D(x=100, y=100),\n    ])\n\nap.save_overall_html(\n    dest_dir_path=\'point2d_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/point2d_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Point2D class constructor API':
    '',

    '<!-- Docstring: apysc._geom.point2d.Point2D.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, x:Union[int, apysc._type.int.Int], y:Union[int, apysc._type.int.Int]) -> None`<hr>\n\n**[Interface summary]** 2-dimensional geometry point.<hr>\n\n**[Parameters]**\n\n- `x`: int or Int\n  - X-coordinate.\n- `y`: int or Int\n  - Y-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=0, y=0),\n...         ap.Point2D(x=0, y=50),\n...         ap.Point2D(x=50, y=25),\n...     ])\n```':  # noqa
    '',

    '':
    '',

    '## x property API':
    '',

    '<!-- Docstring: apysc._geom.point2d.Point2D.x -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** X-coordinate property.<hr>\n\n**[Returns]**\n\n- `x`: Int\n  - X-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> point: ap.Point2D = ap.Point2D(x=50, y=100)\n>>> point.x = ap.Int(150)\n>>> point.x\nInt(150)\n```':  # noqa
    '',

    '':
    '',

    '## y property API':
    '',

    '<!-- Docstring: apysc._geom.point2d.Point2D.y -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Y-coordinate property.<hr>\n\n**[Returns]**\n\n- `y`: Int\n  - Y-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> point: ap.Point2D = ap.Point2D(x=50, y=100)\n>>> point.y = ap.Int(150)\n>>> point.y\nInt(150)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: graphics_line_dash_setting.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics line_dash_setting interface':
    '',

    'This page explains the `Graphics` class `line_dash_setting` property interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `line_dash_setting` property interface updates or gets the instance\'s current line dash setting.':  # noqa
    '',

    '## Basic usage':
    '',

    'The getter or setter interface value becomes (or requires) the `LineDashSetting` instance value.\n\nThe following example sets the 10-pixel dash size and 3-pixel space size to the line:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250, stage_height=100, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=\'#0af\', thickness=5)\n\nline: ap.Line = sprite.graphics.draw_line(\n    x_start=50, y_start=50, x_end=200, y_end=50)\nline.line_dash_setting = ap.LineDashSetting(dash_size=10, space_size=3)\n\nap.save_overall_html(\n    dest_dir_path=\'./graphics_line_dash_setting_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_dash_setting_basic_usage/index.html" width="250" height="100"></iframe>':  # noqa
    '',

    '## line_dash_setting property API':
    '',

    '<!-- Docstring: apysc._display.line_dash_setting_interface.LineDashSettingInterface.line_dash_setting -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line dash setting.<hr>\n\n**[Returns]**\n\n- `line_dash_setting`: LineDashSetting or None\n  - Line dash setting.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=\'#fff\', thickness=10)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50)\n>>> line.line_dash_setting = ap.LineDashSetting(\n...     dash_size=5, space_size=2)\n>>> line.line_dash_setting.dash_size\nInt(5)\n\n>>> line.line_dash_setting.space_size\nInt(2)\n```':  # noqa
    '',

}



"""Dot setting class implementation for a line.
"""

from typing import Union

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.dictionary import Dictionary
from apysc._type.int import Int


class LineDotSetting(Dictionary[str, Int]):
    """
    Dot setting class for a line.

    References
    ----------
    - Graphics line_style interface document
        - https://simon-ritchie.github.io/apysc/graphics_line_style.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color='#fff', thickness=10)
    >>> line: ap.Line = sprite.graphics.draw_line(
    ...     x_start=50, y_start=50, x_end=150, y_end=50)
    >>> line.line_dot_setting = ap.LineDotSetting(dot_size=5)
    >>> line.line_dot_setting.dot_size
    Int(5)
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineDotSetting')
    def __init__(self, dot_size: Union[int, Int]) -> None:
        """
        Dot setting class for a line.

        Parameters
        ----------
        dot_size : int or Int
            Dot size.

        References
        ----------
        - Graphics line_style interface document
            - https://simon-ritchie.github.io/apysc/graphics_line_style.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color='#fff', thickness=10)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50)
        >>> line.line_dot_setting = ap.LineDotSetting(dot_size=5)
        >>> line.line_dot_setting.dot_size
        Int(5)
        """
        import apysc as ap
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_int_from_builtin_val
        from apysc._validation import number_validation
        number_validation.validate_nums_are_int_and_gt_zero(
            nums=[dot_size])
        dot_size_: ap.Int = get_copied_int_from_builtin_val(
            integer=dot_size)
        super(LineDotSetting, self).__init__({'dot_size': dot_size_})

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineDotSetting')
    def dot_size(self) -> Int:
        """
        Get a dot size setting.

        Returns
        -------
        dot_size : Int
            Dot size setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color='#fff', thickness=10)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50)
        >>> line.line_dot_setting = ap.LineDotSetting(dot_size=5)
        >>> line.line_dot_setting.dot_size
        Int(5)
        """
        return self['dot_size']._copy()



"""If branch instruction implementations.
"""

from typing import Any
from typing import Dict
from typing import Optional

from apysc._branch.if_base import IfBase
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean


class If(IfBase):
    """
    A class to append if branch instruction expression.

    References
    ----------
    - If document
        - https://simon-ritchie.github.io/apysc/if.html
    - Each branch instruction class's scope variables reverting setting
        - https://simon-ritchie.github.io/apysc/branch_instruction_variables_reverting_setting.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> int_val: ap.Int = ap.Int(10)
    >>> condition: ap.Boolean = int_val >= 10
    >>> with ap.If(condition):
    ...     ap.trace('Int value is greater than equal 10!')
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='If')
    def __init__(
            self,
            condition: Optional[Boolean],
            *,
            locals_: Optional[Dict[str, Any]] = None,
            globals_: Optional[Dict[str, Any]] = None) -> None:
        """
        A class to append if branch instruction expression.

        Parameters
        ----------
        condition : Boolean or None
            Boolean value to be used for judgment.
        locals_ : dict or None, default None
            Current scope's local variables. Set locals()
            value to this argument. If specified, this interface
            reverts all local scope VariableNameInterface
            variables (like Int, Sprite) at the end of an `If`
            scope. This setting is useful when you don't want to
            update each variable by implementing the `If` scope.
        globals_ : dict or None, default None
            Current scope's global variables. Set globals()
            value to this argument. This setting works
            the same way as the locals_ argument.

        References
        ----------
        - If document
            - https://simon-ritchie.github.io/apysc/if.html
        - Each branch instruction class's scope variables reverting setting
            - https://simon-ritchie.github.io/apysc/branch_instruction_variables_reverting_setting.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> int_val: ap.Int = ap.Int(10)
        >>> condition: ap.Boolean = int_val >= 10
        >>> with ap.If(condition):
        ...     ap.trace('Int value is greater than equal 10!')
        """
        super(If, self).__init__(
            condition=condition, locals_=locals_, globals_=globals_)

    def _append_enter_expression(self) -> None:
        """
        Append if branch instruction starting expression.
        """
        import apysc as ap
        if self._condition is None:
            raise ValueError(
                'If expression\'s condition argument can\'t be set None.')
        expression: str = (
            f'if ({self._condition.variable_name}) {{'
        )
        ap.append_js_expression(expression=expression)

    def _set_last_scope(self) -> None:
        """
        Set expression last scope value.
        """
        from apysc._expression import last_scope
        from apysc._expression.last_scope import LastScope
        last_scope.set_last_scope(value=LastScope.IF)



"""This module is for the translation mapping data of the
following document:

Document file: array_append_and_push.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array class append and push interfaces':
    '',

    'This page explains the `Array` class `append` and `push` method interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `append` and `push` method interfaces append any value to the end of an array. These interfaces behave the same (`append` is similar to the Python built-in and the `push` interface is similar to the JavaScript).':  # noqa
    '',

    '## Basic usage':
    '',

    'The `append` and `push` methods require the first argument of the `value`\\.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2])\narr.append(value=3)\nassert arr == [1, 2, 3]\n\narr.push(value=4)\nassert arr == [1, 2, 3, 4]\n```':  # noqa
    '',

    '':
    '',

    '## append API':
    '',

    '<!-- Docstring: apysc._type.array.Array.append -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `append(self, value:~T) -> None`<hr>\n\n**[Interface summary]** Add any value to the end of this array. This method behaves the same `push` method.<hr>\n\n**[Parameters]**\n\n- `value`: *\n  - Any value to append.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.append(4)\n>>> arr\nArray([1, 2, 3, 4])\n```':  # noqa
    '',

    '':
    '',

    '## push API':
    '',

    '<!-- Docstring: apysc._type.array.Array.push -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `push(self, value:~T) -> None`<hr>\n\n**[Interface summary]** Add any value to the end of this array. This interface behaves the same as the `append` method.<hr>\n\n**[Parameters]**\n\n- `value`: *\n  - Any value to append.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.push(4)\n>>> arr\nArray([1, 2, 3, 4])\n```':  # noqa
    '',

}



from apysc._type.int import Int  # isort:skip # noqa
from apysc._type.number import Number  # isort:skip # noqa
from apysc._type.number import Number as Float  # isort:skip # noqa
from apysc._type.boolean import Boolean  # isort:skip # noqa
from apysc._type.boolean import Boolean as Bool  # isort:skip # noqa
from apysc._type.string import String  # isort:skip # noqa
from apysc._type.array import Array  # isort:skip # noqa
from apysc._type.dictionary import Dictionary  # isort:skip # noqa
from apysc._type.any_value import AnyValue  # isort:skip # noqa
from apysc._branch._if import If  # isort:skip # noqa
from apysc._branch._elif import Elif  # isort:skip # noqa
from apysc._branch._else import Else  # isort:skip # noqa
from apysc._loop._for import For  # isort:skip # noqa
from apysc._loop._continue import Continue  # isort:skip # noqa
from apysc._display.display_object import DisplayObject  # isort:skip # noqa
from apysc._display._document import document  # isort:skip # noqa
from apysc._display.sprite import Sprite  # isort:skip # noqa
from apysc._display.graphics import Graphics  # isort:skip # noqa
from apysc._display.stage import Stage  # isort:skip # noqa
from apysc._display.stage import get_stage  # isort:skip # noqa
from apysc._display.rectangle import Rectangle  # isort:skip # noqa
from apysc._display.circle import Circle  # isort:skip # noqa
from apysc._display.ellipse import Ellipse  # isort:skip # noqa
from apysc._display.line import Line  # isort:skip # noqa
from apysc._display.polyline import Polyline  # isort:skip # noqa
from apysc._display.polygon import Polygon  # isort:skip # noqa
from apysc._display.line_caps import LineCaps  # isort:skip # noqa
from apysc._display.line_joints import LineJoints  # isort:skip # noqa
from apysc._display.line_dot_setting import LineDotSetting  # isort:skip # noqa
from apysc._display.line_dash_setting import LineDashSetting  # isort:skip # noqa
from apysc._display.line_round_dot_setting import LineRoundDotSetting  # isort:skip # noqa
from apysc._display.line_dash_dot_setting import LineDashDotSetting  # isort:skip # noqa
from apysc._display.path import Path  # isort:skip # noqa
from apysc._geom.point2d import Point2D  # isort:skip # noqa
from apysc._geom.path_label import PathLabel  # isort:skip # noqa
from apysc._geom.path_data_base import PathDataBase  # isort:skip # noqa
from apysc._geom.path_move_to import PathMoveTo  # isort:skip # noqa
from apysc._geom.path_line_to import PathLineTo  # isort:skip # noqa
from apysc._geom.path_horizontal import PathHorizontal  # isort:skip # noqa
from apysc._geom.path_vertical import PathVertical  # isort:skip # noqa
from apysc._geom.path_close import PathClose  # isort:skip # noqa
from apysc._geom.path_bezier_2d import PathBezier2D  # isort:skip # noqa
from apysc._geom.path_bezier_2d_continual import PathBezier2DContinual  # isort:skip # noqa
from apysc._geom.path_bezier_3d import PathBezier3D  # isort:skip # noqa
from apysc._geom.path_bezier_3d_continual import PathBezier3DContinual  # isort:skip # noqa
from apysc._geom.path_data import PathData  # isort:skip # noqa
from apysc._event.event import Event  # isort:skip # noqa
from apysc._event.mouse_event import MouseEvent  # isort:skip # noqa
from apysc._event.wheel_event import WheelEvent  # isort:skip # noqa
from apysc._event.timer_event import TimerEvent  # isort:skip # noqa
from apysc._event.animation_event import AnimationEvent  # isort:skip # noqa
from apysc._event.mouse_event_type import MouseEventType  # isort:skip # noqa
from apysc._console._trace import trace  # isort:skip # noqa
from apysc._console.assertion import assert_equal  # isort:skip # noqa
from apysc._console.assertion import assert_not_equal  # isort:skip # noqa
from apysc._console.assertion import assert_true  # isort:skip # noqa
from apysc._console.assertion import assert_false  # isort:skip # noqa
from apysc._console.assertion import assert_arrays_equal  # isort:skip # noqa
from apysc._console.assertion import assert_arrays_not_equal  # isort:skip # noqa
from apysc._console.assertion import assert_dicts_equal  # isort:skip # noqa
from apysc._console.assertion import assert_dicts_not_equal  # isort:skip # noqa
from apysc._console.assertion import assert_defined  # isort:skip # noqa
from apysc._console.assertion import assert_undefined  # isort:skip # noqa
from apysc._event.document_mouse_wheel_interface import bind_wheel_event_to_document  # isort:skip # noqa
from apysc._event.document_mouse_wheel_interface import unbind_wheel_event_all_from_document  # isort:skip # noqa
from apysc._event.document_mouse_wheel_interface import unbind_wheel_event_from_document  # isort:skip # noqa
from apysc._html.exporter import save_overall_html  # isort:skip # noqa
from apysc._expression.expression_data_util import append_js_expression  # isort:skip # noqa
from apysc._jupyter.jupyter_util import display_on_jupyter  # isort:skip # noqa
from apysc._jupyter.jupyter_util import display_on_colaboratory  # isort:skip # noqa
from apysc._time.fps import FPS  # isort:skip # noqa
from apysc._time.timer import Timer  # isort:skip # noqa
from apysc._html.debug_mode import set_debug_mode  # isort:skip # noqa
from apysc._html.debug_mode import unset_debug_mode  # isort:skip # noqa
from apysc._html.debug_mode import is_debug_mode  # isort:skip # noqa
from apysc._html.debug_mode import add_debug_info_setting  # isort:skip # noqa
from apysc._type._return import Return  # isort:skip # noqa
from apysc._animation.easing import Easing  # isort:skip # noqa
from apysc._animation.animation_base import AnimationBase  # isort:skip # noqa
from apysc._animation.animation_move import AnimationMove  # isort:skip # noqa
from apysc._animation.animation_x import AnimationX  # isort:skip # noqa
from apysc._animation.animation_y import AnimationY  # isort:skip # noqa
from apysc._animation.animation_cx import AnimationCx  # isort:skip # noqa
from apysc._animation.animation_cy import AnimationCy  # isort:skip # noqa
from apysc._animation.animation_width import AnimationWidth  # isort:skip # noqa
from apysc._animation.animation_height import AnimationHeight  # isort:skip # noqa
from apysc._animation.animation_width_for_ellipse import AnimationWidthForEllipse  # isort:skip # noqa
from apysc._animation.animation_height_for_ellipse import AnimationHeightForEllipse  # isort:skip # noqa
from apysc._animation.animation_radius import AnimationRadius  # isort:skip # noqa
from apysc._animation.animation_fill_alpha import AnimationFillAlpha  # isort:skip # noqa
from apysc._animation.animation_fill_color import AnimationFillColor  # isort:skip # noqa
from apysc._animation.animation_line_color import AnimationLineColor  # isort:skip # noqa
from apysc._animation.animation_line_alpha import AnimationLineAlpha  # isort:skip # noqa
from apysc._animation.animation_line_thickness import AnimationLineThickness  # isort:skip # noqa
from apysc._animation.animation_skew_x import AnimationSkewX  # isort:skip # noqa
from apysc._animation.animation_skew_y import AnimationSkewY  # isort:skip # noqa
from apysc._animation.animation_rotation_around_center import AnimationRotationAroundCenter  # isort:skip # noqa
from apysc._animation.animation_rotation_around_point import AnimationRotationAroundPoint  # isort:skip # noqa
from apysc._animation.animation_scale_x_from_center import AnimationScaleXFromCenter  # isort:skip # noqa
from apysc._animation.animation_scale_y_from_center import AnimationScaleYFromCenter  # isort:skip # noqa
from apysc._animation.animation_scale_x_from_point import AnimationScaleXFromPoint  # isort:skip # noqa
from apysc._animation.animation_scale_y_from_point import AnimationScaleYFromPoint  # isort:skip # noqa
from apysc._animation.animation_parallel import AnimationParallel  # isort:skip # noqa

__version__: str = '0.65.23'



"""Class implementation for number value interface.
"""

from typing import Any
from typing import Dict
from typing import Generic
from typing import TypeVar
from typing import Union

from apysc._event.custom_event_interface import CustomEventInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.copy_interface import CopyInterface
from apysc._type.revert_interface import RevertInterface
from apysc._type.variable_name_interface import VariableNameInterface

_NumType = Union[int, float, 'NumberValueInterface']
_V = TypeVar('_V', int, float)
_T = TypeVar('_T', bound='NumberValueInterface')


class NumberValueInterface(
        CopyInterface, RevertInterface, CustomEventInterface,
        Generic[_V, _T]):

    _initial_value: _NumType
    _value: _V

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __init__(
            self, *, value: _NumType,
            type_name: str) -> None:
        """
        Class for number value interface.

        Parameters
        ----------
        value : NumberValueInterface or int or float
            Initial number value.
        type_name : str
            This instance expression's type name (e.g., int, number).
        """
        from apysc._validation import number_validation
        number_validation.validate_num(
            num=value)  # type: ignore
        self._initial_value = value
        if isinstance(value, NumberValueInterface):
            value_: _V = value._value
        else:
            value_ = value  # type: ignore
        self._value = value_
        self._type_name = type_name

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def append_constructor_expression(self) -> None:
        """
        Append current value's constructor expression.
        """
        import apysc as ap
        if isinstance(self._initial_value, NumberValueInterface):
            value_: Union[int, float, str] = \
                self._initial_value.variable_name
        else:
            value_ = self._value
        expression: str = (
            f'var {self.variable_name} = {value_};'
        )
        ap.append_js_expression(expression=expression)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def value(self) -> _NumType:
        """
        Get a current number value.

        Returns
        -------
        value : int or float
            Current number value.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/fundamental_data_classes_value_interface.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> int_val: ap.Int = ap.Int(10)
        >>> int_val.value
        10

        >>> int_val.value = 20
        >>> int_val.value
        20

        >>> int_val.value = ap.Int(30)
        >>> int_val.value
        30
        """
        return self._value

    @value.setter
    def value(self, value: _NumType) -> None:
        """
        Set number value.

        Parameters
        ----------
        value : NumberValueInterface or int or float
            Any number value to set.

        References
        ----------
        apysc fundamental data classes value interface
            https://simon-ritchie.github.io/apysc/fundamental_data_classes_value_interface.html  # noqa
        """
        from apysc._html.debug_mode import DebugInfo
        with DebugInfo(
                callable_='value', args=[value], kwargs={},
                module_name=__name__,
                class_name=NumberValueInterface.__name__):
            self._set_value_and_skip_expression_appending(value=value)
            if isinstance(value, NumberValueInterface):
                self._append_value_setter_expression(value=value)
            else:
                self._append_value_setter_expression(value=self._value)

    def _set_value_and_skip_expression_appending(
            self, *, value: _NumType) -> None:
        """
        Update value attribute and skip expression appending.

        Parameters
        ----------
        value : NumberValueInterface or int or float
            Any number value to set.
        """
        from apysc._validation import number_validation
        number_validation.validate_num(
            num=value)  # type: ignore
        if isinstance(value, NumberValueInterface):
            value_: _V = value._value
        else:
            value_ = value  # type: ignore
        self._value = value_

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_value_setter_expression(
            self, *, value: _NumType) -> None:
        """
        Append value's setter-expresion.

        Parameters
        ----------
        value : NumberValueInterface or int or float
            Any number value to set.
        """
        import apysc as ap
        if isinstance(value, NumberValueInterface):
            right_value: Union[str, int, float] = value.variable_name
        else:
            right_value = value  # type: ignore
        expression: str = (
            f'{self.variable_name} = {right_value};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __add__(self, other: _NumType) -> Any:
        """
        Method for addition.

        Parameters
        ----------
        other : NumberValueInterface or int or float
            Other value to add.

        Returns
        -------
        result : NumberValueInterface
            Addition result value.
        """
        if isinstance(other, NumberValueInterface):
            value: _NumType = self._value + other._value
        else:
            value = self._value + other  # type: ignore
        result: NumberValueInterface = self._copy()
        result._set_value_and_skip_expression_appending(value=value)
        self._append_addition_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_addition_expression(
            self, *, result: VariableNameInterface,
            other: _NumType) -> None:
        """
        Append addition expression.

        Parameters
        ----------
        result : NumberValueInterface
            Addition result value.
        other : NumberValueInterface or int or float
            Other value to add.
        """
        import apysc as ap
        from apysc._type.value_util import get_value_str_for_expression
        right_value: str = get_value_str_for_expression(value=other)
        expression: str = (
            f'var {result.variable_name} = '
            f'{self.variable_name} + {right_value};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __sub__(self, other: _NumType) -> Any:
        """
        Method for subtraction.

        Parameters
        ----------
        other : NumberValueInterface or int or float
            The other value to subtract.

        Returns
        -------
        result : NumberValueInterface
            Subtraction result value.
        """
        if isinstance(other, NumberValueInterface):
            value: _NumType = self._value - other._value
        else:
            value = self._value - other  # type: ignore
        result: NumberValueInterface = self._copy()
        result._set_value_and_skip_expression_appending(value=value)
        self._append_subtraction_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_subtraction_expression(
            self, *, result: VariableNameInterface,
            other: _NumType) -> None:
        """
        Append subtraction expression.

        Parameters
        ----------
        result : NumberValueInterface
            Subtraction result value.
        other : NumberValueInterface or int or float
            Other value to subtract.
        """
        import apysc as ap
        from apysc._type.value_util import get_value_str_for_expression
        right_value: str = get_value_str_for_expression(value=other)
        expression: str = (
            f'var {result.variable_name} = '
            f'{self.variable_name} - {right_value};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __mul__(self, other: _NumType) -> _T:
        """
        Method for multiplication.

        Parameters
        ----------
        other : NumberValueInterface or int or float
            Other value to multiply.

        Returns
        -------
        result : NumberValueInterface
            Multiplication result value.
        """
        if isinstance(other, NumberValueInterface):
            value: _NumType = self._value * other._value
        else:
            value = self._value * other  # type: ignore
        result: _T = self._copy()
        result._set_value_and_skip_expression_appending(value=value)
        self._append_multiplication_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_multiplication_expression(
            self, *, result: VariableNameInterface,
            other: _NumType) -> None:
        """
        Append multiplication expression.

        Parameters
        ----------
        result : NumberValueInterface
            Multiplication result value.
        other : NumberValueInterface or int or float
            Other value to multiply.
        """
        import apysc as ap
        from apysc._type.value_util import get_value_str_for_expression
        right_value: str = get_value_str_for_expression(value=other)
        expression: str = (
            f'var {result.variable_name} = '
            f'{self.variable_name} * {right_value};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __truediv__(self, other: _NumType) -> Any:
        """
        Method for true division (returns floating-point number).

        Parameters
        ----------
        other : NumberValueInterface or int or float
            Other value for true-division.

        Returns
        -------
        result : Number
            True division result value.
        """
        import apysc as ap
        result: ap.Number = ap.Number(value=self)
        if isinstance(other, NumberValueInterface):
            value: _NumType = result._value / other._value
        else:
            value = result._value / other  # type: ignore
        result._set_value_and_skip_expression_appending(value=value)
        self._append_true_division_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_true_division_expression(
            self, *, result: VariableNameInterface,
            other: _NumType) -> None:
        """
        Append true division expression.

        Parameters
        ----------
        result : NumberValueInterface
            True division result value.
        other : NumberValueInterface or int or float
            Other value for true division.
        """
        import apysc as ap
        from apysc._type.value_util import get_value_str_for_expression
        right_value: str = get_value_str_for_expression(value=other)
        expression: str = (
            f'{result.variable_name} = {self.variable_name} / '
            f'{right_value};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __floordiv__(self, other: _NumType) -> Any:
        """
        Method for floor division (return integer).

        Parameters
        ----------
        other : NumberValueInterface or int or float
            Other value for floor division.

        Returns
        -------
        result : Int
            Floor division result value.
        """
        import apysc as ap
        result: ap.Int = ap.Int(value=self)
        if isinstance(other, NumberValueInterface):
            value: _NumType = self._value // other._value
        else:
            value = self._value // other  # type: ignore
        result._set_value_and_skip_expression_appending(value=value)
        self._append_floor_division_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_floor_division_expression(
            self, *, result: VariableNameInterface,
            other: _NumType) -> None:
        """
        Append floor division expression.

        Parameters
        ----------
        result : NumberValueInterface
            Floor division result value.
        other : NumberValueInterface or int or float
            Other value for floor division.
        """
        import apysc as ap
        from apysc._type.value_util import get_value_str_for_expression
        right_value: str = get_value_str_for_expression(value=other)
        expression: str = (
            f'{result.variable_name} = '
            f'parseInt({self.variable_name} / {right_value});'
        )
        ap.append_js_expression(expression=expression)

    _incremental_calc_prev_name: str = ''

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_incremental_calc_substitution_expression(self) -> None:
        """
        Append an incremental calculation's substitution expression.
        Each interface call this method.
        """
        from apysc._expression import expression_variables_util
        expression_variables_util.\
            append_substitution_expression_with_names(
                left_variable_name=self._incremental_calc_prev_name,
                right_variable_name=self.variable_name)
        self._incremental_calc_prev_name = ''

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __iadd__(self, other: _NumType) -> _T:
        """
        Method for incremental addition.

        Parameters
        ----------
        other : NumberValueInterface or int or float
            Other value for` incremental addition.

        Returns
        -------
        result : NumberValueInterface
            Incremental addition result value.
        """
        from apysc._expression import expression_variables_util
        self._incremental_calc_prev_name = \
            self._get_previous_variable_name()
        result: _T = self + other
        expression_variables_util.append_substitution_expression(
            left_value=self, right_value=result)
        result.variable_name = self.variable_name
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __isub__(self, other: _NumType) -> Any:
        """
        Method for incremental subtraction.

        Parameters
        ----------
        other : NumberValueInterface or int or float
            Other value for incremental subtraction.

        Returns
        -------
        result : NumberValueInterface
            Incremental subtraction result value.
        """
        from apysc._expression import expression_variables_util
        self._incremental_calc_prev_name = \
            self._get_previous_variable_name()
        result: NumberValueInterface = self - other
        expression_variables_util.append_substitution_expression(
            left_value=self, right_value=result)
        result.variable_name = self.variable_name
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __imul__(self, other: _NumType) -> _T:
        """
        Method for incremental multiplication.

        Parameters
        ----------
        other : NumberValueInterface or int or float
            Other value for incremental multiplication.

        Returns
        -------
        result : NumberValueInterface
            Incremental multiplication result value.
        """
        from apysc._expression import expression_variables_util
        self._incremental_calc_prev_name = \
            self._get_previous_variable_name()
        result: _T = self * other
        expression_variables_util.append_substitution_expression(
            left_value=self, right_value=result)
        result.variable_name = self.variable_name
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __itruediv__(self, other: _NumType) -> Any:
        """
        Method for incremental true division.

        Parameters
        ----------
        other : NumberValueInterface or int or float
            Other value for incremental true division.

        Returns
        -------
        result : NumberValueInterface
            The other value for incremental-true division.
        """
        from apysc._expression import expression_variables_util
        self._incremental_calc_prev_name = \
            self._get_previous_variable_name()
        result: NumberValueInterface = self / other
        result._incremental_calc_prev_name = \
            self._incremental_calc_prev_name
        expression_variables_util.append_substitution_expression(
            left_value=self, right_value=result)
        result.variable_name = self.variable_name
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __mod__(self, other: _NumType) -> _T:
        """
        Method for the modulo operation.

        Parameters
        ----------
        other : NumberValueInterface or int or float
            The other value to use in the modulo operation.

        Returns
        -------
        result : NumberValueInterface
            Modulo operation result value.
        """
        if isinstance(other, NumberValueInterface):
            value: Union[int, float] = self._value % other._value
        else:
            value = self._value % other  # type: ignore
        result: _T = self._copy()
        result._set_value_and_skip_expression_appending(value=value)
        self._append_modulo_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_modulo_expression(
            self, *, result: VariableNameInterface,
            other: _NumType) -> None:
        """
        Append a module expression.

        Parameters
        ----------
        result : NumberValueInterface
            Modulo operation result value.
        other : NumberValueInterface or int or float
            The other value to use in the modulo operation.
        """
        import apysc as ap
        from apysc._type.value_util import get_value_str_for_expression
        right_value: str = get_value_str_for_expression(value=other)
        expression: str = (
            f'var {result.variable_name} = '
            f'{self.variable_name} % {right_value};'
        )
        ap.append_js_expression(expression=expression)

    def __str__(self) -> str:
        """
        String conversion method.

        Returns
        -------
        string : str
            Converted value string.
        """
        if not hasattr(self, '_value'):
            return '0'
        return str(self._value)

    def __int__(self) -> int:
        """
        Integer conversion method.

        Returns
        -------
        integer : int
            Converted integer value.
        """
        return int(self._value)

    def __float__(self) -> float:
        """
        Float conversion method.

        Returns
        -------
        float_ : float
            Converted float value.
        """
        return float(self._value)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __eq__(self, other: Any) -> Any:
        """
        Equal comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. Builtin types, Int,
            and Number class instances are acceptable.

        Returns
        -------
        result : Boolean
            If a specified value is the same amount, this interface
            returns True.
        """
        import apysc as ap
        if isinstance(other, NumberValueInterface):
            result: ap.Boolean = ap.Boolean(self._value == other._value)
        else:
            result = ap.Boolean(self._value == other)
        other = self._convert_other_val_to_int_or_number(other=other)
        if isinstance(other, VariableNameInterface):
            self._append_eq_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _convert_other_val_to_int_or_number(self, *, other: Any) -> Any:
        """
        Convert a specified other value if comparison
        its type is an int or float, then convert it
        to Int or Number.

        Parameters
        ----------
        other : *
            Other comparison value.

        Returns
        -------
        converted_val : *
            Converted value. If an int is specified, this
            interface converts it to an Int. Similarly, if a
            float is specified, this interface converts it
            to a Number value. This interface returns the
            other type directly (not to be converted).
        """
        import apysc as ap
        if isinstance(other, int):
            return ap.Int(other)
        if isinstance(other, float):
            return ap.Number(other)
        return other

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_eq_expression(
            self, *, result: VariableNameInterface,
            other: VariableNameInterface) -> None:
        """
        Append __eq__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameInterface
            The other value to compare.
        """
        import apysc as ap
        expression: str = (
            f'{result.variable_name} = '
            f'{self.variable_name} === {other.variable_name};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __ne__(self, other: Any) -> Any:
        """
        Not equal comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. Builtin types, Int,
            and Number class instances are acceptable.

        Returns
        -------
        result : Boolean
            If a specified value is not the same amount, this interface
            returns True.
        """
        import apysc as ap
        if isinstance(other, NumberValueInterface):
            result: ap.Boolean = ap.Boolean(self._value != other._value)
        else:
            result = ap.Boolean(self._value != other)
        other = self._convert_other_val_to_int_or_number(other=other)
        if isinstance(other, VariableNameInterface):
            self._append_ne_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_ne_expression(
            self, *, result: VariableNameInterface,
            other: VariableNameInterface) -> None:
        """
        Append __ne__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameInterface
            The other value to compare.
        """
        import apysc as ap
        expression: str = (
            f'{result.variable_name} = '
            f'{self.variable_name} !== {other.variable_name};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __lt__(self, other: Any) -> Any:
        """
        Less than comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. Builtin types, Int,
            and Number class instances are acceptable.

        Returns
        -------
        result : Boolean
            If this value is less than a specified value,
            this interface returns True.
        """
        import apysc as ap
        if isinstance(other, NumberValueInterface):
            result: ap.Boolean = ap.Boolean(self._value < other._value)
        else:
            result = ap.Boolean(self._value < other)
        other = self._convert_other_val_to_int_or_number(other=other)
        if isinstance(other, VariableNameInterface):
            self._append_lt_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_lt_expression(
            self, *, result: VariableNameInterface,
            other: VariableNameInterface) -> None:
        """
        Append __lt__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameInterface
            The other value to compare.
        """
        import apysc as ap
        expression: str = (
            f'{result.variable_name} = '
            f'{self.variable_name} < {other.variable_name};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __le__(self, other: Any) -> Any:
        """
        Less than equal comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. Builtin types, Int,
            and Number class instances are acceptable.

        Returns
        -------
        result : Boolean
            If this value is less than or equal to a specified
            value, this interface returns True.
        """
        import apysc as ap
        if isinstance(other, NumberValueInterface):
            result: ap.Boolean = ap.Boolean(self._value <= other._value)
        else:
            result = ap.Boolean(self._value <= other)
        other = self._convert_other_val_to_int_or_number(other=other)
        if isinstance(other, VariableNameInterface):
            self._append_le_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_le_expression(
            self, *, result: VariableNameInterface,
            other: VariableNameInterface) -> None:
        """
        Append __le__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameInterface
            The other value to compare.
        """
        import apysc as ap
        expression: str = (
            f'{result.variable_name} = '
            f'{self.variable_name} <= {other.variable_name};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __gt__(self, other: Any) -> Any:
        """
        Greater than comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. Builtin types, Int,
            and Number class instances are acceptable.

        Returns
        -------
        result : Boolean
            If this value is greater than a specified value,
            this interface returns True.
        """
        import apysc as ap
        if isinstance(other, NumberValueInterface):
            result: ap.Boolean = ap.Boolean(self._value > other._value)
        else:
            result = ap.Boolean(self._value > other)
        other = self._convert_other_val_to_int_or_number(other=other)
        if isinstance(other, NumberValueInterface):
            self._append_gt_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_gt_expression(
            self, *, result: VariableNameInterface,
            other: VariableNameInterface) -> None:
        """
        Append __gt__ expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameInterface
            The other value to compare.
        """
        import apysc as ap
        expression: str = (
            f'{result.variable_name} = '
            f'{self.variable_name} > {other.variable_name};'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def __ge__(self, other: Any) -> Any:
        """
        Greater than equal comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. Builtin types, Int,
            and Number class instances are acceptable.

        Returns
        -------
        result : Boolean
            If this value is greater than or equal to a specified
            value, this interface returns True.
        """
        import apysc as ap
        if isinstance(other, NumberValueInterface):
            result: ap.Boolean = ap.Boolean(self._value >= other._value)
        else:
            result = ap.Boolean(self._value >= other)
        other = self._convert_other_val_to_int_or_number(other=other)
        if isinstance(other, VariableNameInterface):
            self._append_ge_expression(result=result, other=other)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='NumberValueInterface')
    def _append_ge_expression(
            self, *, result: VariableNameInterface,
            other: VariableNameInterface) -> None:
        """
        Append __ge__ expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameInterface
            The other value to compare.
        """
        import apysc as ap
        expression: str = (
            f'{result.variable_name} = '
            f'{self.variable_name} >= {other.variable_name};'
        )
        ap.append_js_expression(expression=expression)

    _value_snapshots: Dict[str, _V]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name='_value_snapshots',
            value=self._value, snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._value = self._value_snapshots[snapshot_name]



"""Each types common value utilities.

Mainly following interfaces are defined:

- get_value_str_for_expression
    - Get a value string for expression.
- get_copy
    - Get a copy of a specified instance if it is an instance
    of CopyInterface.
"""

from typing import Any
from typing import Dict
from typing import List
from typing import TypeVar
from typing import Union

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array


def get_value_str_for_expression(*, value: Any) -> str:
    """
    Get a value string for expression.

    Parameters
    ----------
    value : *
        Any value to convert to string.

    Returns
    -------
    value_str : str
        String for expression. If a value is an instance of
        VariableNameInterface, this interface returns
        a variable's name. Otherwise, this interface returns
        a string-casted value. A bool value becomes lowercase
        (true or false), and this interface quotes a string
        value by double quotation. This interface converts
        a List or tuple value to a JavaScript Array expression,
        e.g., '[10, "Hello!", true, any_variable]'.
        None becomes NaN.
    """
    from apysc._type.expression_string import ExpressionString
    from apysc._type.variable_name_interface import VariableNameInterface
    if isinstance(value, ExpressionString):
        return value.value
    if isinstance(value, VariableNameInterface):
        return value.variable_name
    if isinstance(value, bool):
        return str(value).lower()
    if isinstance(value, str):
        return f'"{value}"'
    if isinstance(value, (list, tuple)):
        value_str: str = _get_value_str_from_iterable(value=value)
        return value_str
    if isinstance(value, dict):
        values_str = _get_value_str_from_dict(value=value)
        return values_str
    if value is None:
        return 'null'
    return str(value)


def _get_value_str_from_dict(*, value: Dict[Any, Any]) -> str:
    """
    Get a value string from a dictionary object.

    Parameters
    ----------
    value : dict
        Dictionary object to convert to string.

    Returns
    -------
    value_str : str
        Converted string, e.g., '{"any_key": 10, "other_key": any_variable}'
    """
    value_str: str = '{'
    for key, value in value.items():
        if value_str != '{':
            value_str += ', '
        _validate_dict_key_type(key=key)
        value_str += f'"{key}": {get_value_str_for_expression(value=value)}'
    value_str += '}'
    return value_str


def _validate_dict_key_type(*, key: Any) -> None:
    """
    Validate whether a dictionary key type is an int or str.

    Parameters
    ----------
    key : *
        Dictionary key to validate.

    Raises
    ------
    TypeError
        If a key-type isn't str or int.
    """
    if isinstance(key, (str, int, float)):
        return
    raise TypeError(
        f'Dictionary key type only supports str and int: {type(key)}')


def _get_value_str_from_iterable(
        *, value: Union[list, tuple, Array]) -> str:
    """
    Get a value string from an iterable object.

    Parameters
    ----------
    value : list or tuple or Array
        Target iterable object.

    Returns
    -------
    value_str : str
        Converted string, e.g., '[10, "Hello!", true, any_variable]'.
    """
    from apysc._type.variable_name_interface import VariableNameInterface
    if isinstance(value, Array):
        value_: List[Any] = value.value  # type: ignore
    else:
        value_ = list(value)  # type: ignore[arg-type]
    value_str: str = '['
    for unit_value in value_:
        if value_str != '[':
            value_str += ', '
        if isinstance(unit_value, VariableNameInterface):
            value_str += f'{unit_value.variable_name}'
            continue
        value_str += get_value_str_for_expression(value=unit_value)
    value_str += ']'
    return value_str


T = TypeVar('T')


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def get_copy(*, value: T) -> T:
    """
    Get a copy of a specified instance if it is an instance
    of CopyInterface.

    Parameters
    ----------
    value : *
        Any value to copy.

    Returns
    -------
    copied : *
        Copied value. If a value is not an instance of
        CopyInterface, this interface returns an argument
        value directly.
    """
    from apysc._type.copy_interface import CopyInterface
    if not isinstance(value, CopyInterface):
        return value
    return value._copy()



"""This module is for the translation mapping data of the
following document:

Document file: num_children.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# num_children interface':
    '',

    'This page explains the container class, like the `Graphics`\\, `Sprite`\\, `Stage`) `num_children` property interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `num_children` property interface returns the integer (`Int`) value of the number of children.':  # noqa
    '',

    '## Notes':
    '',

    'The `Sprite` instance\'s initial children number is 1, not 0 since a sprite instance has a `graphics` child.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `num_children` property returns the number of children (`Int` value). You can use it for the calculation, for instance, coordinates calculation.\n\nThe following example appends a new rectangle when you click the sprite (rectangle) instance. The `num_children` property determines a new rectangle x-coordinate. When clicking a rectangle, this code also displays the current `num_children` property value to the browser console (please press the F12 key).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_sprite_click(\n        e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    rectangle_x: ap.Int = (sprite.num_children - 1) * 100 + 50\n    new_rect: ap.Rectangle = sprite.graphics.draw_rect(\n        x=rectangle_x,\n        y=50, width=50, height=50)\n    sprite.add_child(new_rect)\n    ap.trace(\n        \'Current sprite children number:\', sprite.num_children,\n        \'rectangle x:\', rectangle_x)\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=450,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nsprite.add_child(rectangle_1)\nsprite.click(on_sprite_click)\n\nap.save_overall_html(\n    dest_dir_path=\'num_children_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/num_children_basic_usage/index.html" width="450" height="150"></iframe>':  # noqa
    '',

    '## num_children API':
    '',

    '<!-- Docstring: apysc._display.child_interface.ChildInterface.num_children -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a current children\'s number.<hr>\n\n**[Returns]**\n\n- `num_children`: int\n  - Current children number.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\n>>> rectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=150, y=50, width=50, height=50)\n>>> sprite.graphics.num_children\nInt(2)\n```':  # noqa
    '',

}



"""The script to convert and sync each docstring to
markdown files.
"""

import inspect
import os
import re
from types import ModuleType
from typing import Callable
from typing import Dict
from typing import List
from typing import Match
from typing import Optional
from typing import Tuple
from typing import Type


def convert_recursively(*, dir_path: str) -> List[str]:
    """
    Convert each docstring in the specified directory
    to markdown files recursively.

    Parameters
    ----------
    dir_path : str
        Target directory path.

    Returns
    -------
    saved_markdown_file_paths : list of str
        list of saved markdown file paths.
    """
    from apysc._lint_and_doc import lint_and_doc_hash_util
    if not os.path.isdir(dir_path):
        return []
    saved_markdown_file_paths: List[str] = []
    file_or_dir_names: List[str] = os.listdir(dir_path)
    for file_or_dir_name in file_or_dir_names:
        file_or_dir_path: str = os.path.join(
            dir_path, file_or_dir_name)
        if (os.path.isdir(file_or_dir_path)
                and not _is_excluding_dir_path(dir_path=file_or_dir_path)):
            convert_recursively(dir_path=file_or_dir_path)
            continue
        if not file_or_dir_path.endswith('.py'):
            continue
        is_file_updated: bool = lint_and_doc_hash_util.is_file_updated(
            file_path=file_or_dir_path,
            hash_type=lint_and_doc_hash_util.HashType.DOCSTRING_TO_MARKDOWN)
        if not is_file_updated:
            continue
        saved_markdown_file_path: str = _save_markdown(
            module_path=file_or_dir_path)
        saved_markdown_file_paths.append(saved_markdown_file_path)
        lint_and_doc_hash_util.save_target_file_hash(
            file_path=file_or_dir_path,
            hash_type=lint_and_doc_hash_util.HashType.DOCSTRING_TO_MARKDOWN)
    return saved_markdown_file_paths


_EXCLUDING_DIR_PATHS: List[str] = [
    'apysc/_translation/',
]


def _is_excluding_dir_path(*, dir_path: str) -> bool:
    """
    Get a boolean indicating whether a specified directory path
    is a excluding condition or not.

    Parameters
    ----------
    dir_path : str
        A target directory path.

    Returns
    -------
    result : bool
        This interface returns True if a specified directory path
        is excluding condition.
    """
    for excluding_dir_path in _EXCLUDING_DIR_PATHS:
        if excluding_dir_path in dir_path:
            return True
    return False


def _save_markdown(*, module_path: str) -> str:
    """
    Save a specified module's markdown file.

    Parameters
    ----------
    module_path : str
        Target Python module path.

    Returns
    -------
    markdown_file_path : str
        Saved markdown file path.
    """
    from apysc._file import file_util
    markdown: str = _convert_module_docstring_to_markdown(
        module_path=module_path)
    markdown_file_path: str = module_path.replace('.py', '.md', 1)
    if markdown_file_path.startswith('./'):
        markdown_file_path = markdown_file_path.replace('./', '', 1)
    markdown_file_path = os.path.join(
        './docstring_markdowns/', markdown_file_path)
    file_util.save_plain_txt(
        txt=markdown, file_path=markdown_file_path)
    return markdown_file_path


def _convert_module_docstring_to_markdown(
        *, module_path: str) -> str:
    """
    Convert a specified module's docstring to a markdown string.

    Parameters
    ----------
    module_path : str
        Target Python module path.

    Returns
    -------
    markdown : str
        Converted markdown string.
    """
    from apysc._file import file_util
    from apysc._file import module_util
    module: ModuleType = module_util.read_target_path_module(
        module_path=module_path)
    module_str: str = file_util.read_txt(file_path=module_path)
    markdown: str = f'# `{module.__name__}` docstrings'
    markdown = _append_module_docstring_to_markdown(
        markdown=markdown,
        docstring=module.__doc__)

    toplevel_functions: List[Callable] = _get_module_toplevel_functions(
        module=module)
    for toplevel_function in toplevel_functions:
        if f'def {toplevel_function.__name__}(' not in module_str:
            continue
        markdown = _append_toplevel_function_docstring_to_markdown(
            markdown=markdown, toplevel_function=toplevel_function)

    toplevel_classes: List[Type] = _get_toplevel_classes(
        module=module)
    for toplevel_class in toplevel_classes:
        match: Optional[Match] = re.search(
            pattern=(
                rf'class {toplevel_class.__name__}[:\(]'
            ),
            string=module_str)
        if match is None:
            continue
        markdown = _append_toplevel_class_docstring_to_markdown(
            markdown=markdown, toplevel_class=toplevel_class,
            module_str=module_str)

    return markdown


def _append_toplevel_class_docstring_to_markdown(
        *, markdown: str, toplevel_class: Type,
        module_str: str) -> str:
    """
    Append a top-level class docstring to a specified
    markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    toplevel_class : Type
        Target top-level class.
    module_str : str
        Target Python module's string.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    markdown += f'\n\n## `{toplevel_class.__name__}` class docstring'
    if toplevel_class.__doc__ is not None:
        markdown = _append_each_section_to_markdown(
            markdown=markdown, docstring=toplevel_class.__doc__)

    methods: List[Callable] = _get_methods_from_class(
        class_=toplevel_class)
    for method in methods:
        if method.__doc__ is None:
            continue
        if f'    def {method.__name__}(' not in module_str:
            continue
        markdown += f'\n\n### `{method.__name__}` method docstring'
        markdown = _append_each_section_to_markdown(
            markdown=markdown, docstring=method.__doc__)

    return markdown


def _get_methods_from_class(*, class_: Type) -> List[Callable]:
    """
    Get methods from a specified class.

    Parameters
    ----------
    class_ : Type
        Target class.

    Returns
    -------
    methods : list of Callable
        Extracted methods.
    """
    members: List[Tuple[str, Callable]] = inspect.getmembers(
        class_, predicate=callable)
    methods: List[Callable] = []
    excluding_target_builtin_methods_dict: Dict[str, str] = \
        _get_excluding_target_builtin_methods()
    for _, method in members:
        if method.__doc__ is None:
            continue
        builtin_docstring: str = excluding_target_builtin_methods_dict.get(
            method.__name__, '')
        if method.__doc__ == builtin_docstring or method.__name__ == 'type':
            continue
        methods.append(method)
    return methods


def _get_excluding_target_builtin_methods() -> Dict[str, str]:
    """
    Get an excluding target built-in methods' docstring
    values dict.

    Returns
    -------
    excluding_target_builtin_methods_dict : dict
        A dictionary which has builtin method name's keys
        and docstring values.
    """

    class _EmptyClass:
        pass

    members: List[Tuple[str, Callable]] = inspect.getmembers(
        _EmptyClass, predicate=callable)
    excluding_target_builtin_methods_dict: Dict[str, str] = {}
    for method_name, method in members:
        if method.__doc__ is None:
            continue
        excluding_target_builtin_methods_dict[method_name] = method.__doc__
    return excluding_target_builtin_methods_dict


def _get_toplevel_classes(*, module: ModuleType) -> List[Type]:
    """
    Get top-level classes from a specified module.

    Parameters
    ----------
    module : ModuleType
        Target module.

    Returns
    -------
    toplevel_classes : list of Type
        Top-level classes.
    """
    members: List[Tuple[str, Type]] = inspect.getmembers(
        module, predicate=inspect.isclass)
    toplevel_classes: List[Type] = [class_ for _, class_ in members]
    return toplevel_classes


def _append_toplevel_function_docstring_to_markdown(
        *, markdown: str,
        toplevel_function: Callable) -> str:
    """
    Append a top-level function docstring to a specified
    markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    toplevel_function : Callable
        Target top-level functions.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    if toplevel_function.__doc__ is None:
        return markdown
    if markdown != '':
        markdown += '\n\n'
    markdown += f'## `{toplevel_function.__name__}` function docstring'
    markdown = _append_each_section_to_markdown(
        markdown=markdown,
        docstring=toplevel_function.__doc__,
    )
    return markdown


def _append_each_section_to_markdown(
        markdown: str, docstring: str) -> str:
    """
    Append each docstring section to a specified markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    docstring : str
        Target docstring.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    from apysc._lint_and_doc import docstring_util
    from apysc._lint_and_doc.docstring_util import Example
    from apysc._lint_and_doc.docstring_util import Parameter
    from apysc._lint_and_doc.docstring_util import Raise
    from apysc._lint_and_doc.docstring_util import Reference
    from apysc._lint_and_doc.docstring_util import Return
    summary: str = docstring_util.extract_summary_from_docstring(
        docstring=docstring)
    parameters: List[Parameter] = \
        docstring_util.extract_param_or_rtn_values_from_docstring(
            target_type=Parameter, docstring=docstring)
    returns: List[Return] = \
        docstring_util.extract_param_or_rtn_values_from_docstring(
            target_type=Return, docstring=docstring)
    raises: List[Raise] = docstring_util.extract_raise_values_from_docstring(
        docstring=docstring)
    notes: str = docstring_util.extract_notes_from_docstring(
        docstring=docstring)
    examples: List[Example] = \
        docstring_util.extract_example_values_from_docstring(
            docstring=docstring)
    references: List[Reference] = \
        docstring_util.extract_reference_values_from_docstring(
            docstring=docstring)

    markdown = docstring_util.append_summary_to_markdown(
        markdown=markdown, summary=summary,
        heading_label='')
    markdown = docstring_util.append_params_or_rtns_to_markdown(
        markdown=markdown, params_or_rtns=parameters)
    markdown = docstring_util.append_params_or_rtns_to_markdown(
        markdown=markdown, params_or_rtns=returns)
    markdown = docstring_util.append_raises_to_markdown(
        markdown=markdown, raises=raises)
    markdown = docstring_util.append_notes_to_markdown(
        markdown=markdown, notes=notes)
    markdown = docstring_util.append_examples_to_markdown(
        markdown=markdown, examples=examples)
    markdown = docstring_util.append_references_to_markdown(
        markdown=markdown, references=references)
    markdown = docstring_util.remove_trailing_hr_tag(markdown=markdown)
    return markdown


def _get_module_toplevel_functions(*, module: ModuleType) -> List[Callable]:
    """
    Get top-level functions from a specified module.

    Parameters
    ----------
    module : ModuleType
        Target module.

    Returns
    -------
    toplevel_functions : list of Callable
        Top-level functions.
    """
    members: List[Tuple[str, Callable]] = inspect.getmembers(
        module, predicate=inspect.isfunction)
    toplevel_functions: List[Callable] = [
        function for _, function in members]
    return toplevel_functions


def _append_module_docstring_to_markdown(
        *, markdown: str,
        docstring: Optional[str]) -> str:
    """
    Append a module description docstring to a
    specified markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    docstring : str or None
        Target module description docstring.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    from apysc._lint_and_doc import docstring_util
    from apysc._lint_and_doc.docstring_util import Reference
    if docstring is None:
        return markdown
    summary: str = docstring_util.extract_summary_from_docstring(
        docstring=docstring)
    notes: str = docstring_util.extract_notes_from_docstring(
        docstring=docstring)
    references: List[Reference] = docstring_util.\
        extract_reference_values_from_docstring(docstring=docstring)

    if markdown != '':
        markdown += '\n\n'
    markdown += '## Module summary'
    markdown = docstring_util.append_summary_to_markdown(
        markdown=markdown, summary=summary,
        heading_label='')
    markdown = docstring_util.append_notes_to_markdown(
        markdown=markdown, notes=notes)
    markdown = docstring_util.append_references_to_markdown(
        markdown=markdown, references=references)
    markdown = docstring_util.remove_trailing_hr_tag(markdown=markdown)
    return markdown



"""This module is for the translation mapping data of the
following document:

Document file: timer_complete.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Timer class timer_complete interface':
    '',

    'This page explains the `Timer` class `timer_complete` method interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `timer_complete` method interface binds a new handler that a timer calls when it is complete. For instance, if the `repeat_count` argument is 100, it calls this handler when a timer count reaches 100 times.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `timer_complete` method has the same interface as the other event binding interface (arguments of the `handler` callable and `options` dictionary).\n\nThe following example starts the first timer (rotating the left-side rectangle) when you click the rectangle. If that one completes, then the second timer starts:':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectsOptions(TypedDict):\n    rectangle_1: ap.Rectangle\n    rectangle_2: ap.Rectangle\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_click(e: ap.MouseEvent[ap.Sprite], options: _RectsOptions) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.unbind_click(on_click)\n    rectangle_1: ap.Rectangle = options[\'rectangle_1\']\n    rectangle_2: ap.Rectangle = options[\'rectangle_2\']\n    options_: _RectOptions = {\'rectangle\': rectangle_1}\n    timer_1: ap.Timer = ap.Timer(\n        handler=on_timer, delay=ap.FPS.FPS_60, repeat_count=90,\n        options=options_)\n    options_ = {\'rectangle\': rectangle_2}\n    timer_1.timer_complete(\n        handler=on_timer_1_complete, options=options_)\n    timer_1.start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.rotation_around_center += 1\n\n\ndef on_timer_1_complete(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the first time calls when completed.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_2: ap.Rectangle = options[\'rectangle\']\n    options_: _RectOptions = {\'rectangle\': rectangle_2}\n    timer_2: ap.Timer = ap.Timer(\n        handler=on_timer, delay=ap.FPS.FPS_60, repeat_count=90,\n        options=options_)\n    timer_2.start()\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\noptions: _RectsOptions = {\n    \'rectangle_1\': rectangle_1, \'rectangle_2\': rectangle_2}\nsprite.click(handler=on_click, options=options)\n\nap.save_overall_html(\n    dest_dir_path=\'timer_complete_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/timer_complete_basic_usage/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## timer_complete API':
    '',

    '<!-- Docstring: apysc._time.timer.Timer.timer_complete -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `timer_complete(self, handler:Callable[[_ForwardRef(\'timer_event.TimerEvent\'), ~_O2], NoneType], *, options:Union[~_O2, NoneType]=None) -> str`<hr>\n\n**[Interface summary]** Add a timer complete event listener setting.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - A callable that a timer calls when complete.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to be passed to a handler.\n\n<hr>\n\n**[Returns]**\n\n- `name`: str\n  - Handler\'s name.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options[\'rectangle\']\n...     rectangle.x += 1\n>>> def on_timer_complete(\n...         e: ap.TimerEvent, options: RectOptions) -> None:\n...     ap.trace(\'Timer completed!\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> options: RectOptions = {\'rectangle\': rectangle}\n>>> timer: ap.Timer = ap.Timer(\n...     on_timer, delay=33.3, options=options)\n>>> _ = timer.timer_complete(on_timer_complete)\n>>> timer.start()\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [About the handler options’ type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

}



"""`trace` (console.log expression) interface implementations
"""

from typing import Any
from typing import List

from apysc._html.debug_mode import add_debug_info_setting


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def trace(*args: Any) -> None:
    """
    Display arguments information to console.
    This function saves a JavaScript `console.log` expression.

    Parameters
    ----------
    *args : list
        Any arguments to display to console.

    References
    ----------
    - Trace interface document
        - https://simon-ritchie.github.io/apysc/trace.html

    Examples
    --------
    >>> import apysc as ap
    >>> int_val: ap.Int = ap.Int(10)
    >>> ap.trace('Int value is:', int_val)
    """
    import apysc as ap
    from apysc._string import string_util
    from apysc._type.variable_name_interface import VariableNameInterface

    expression: str = 'console.log('
    arg_strs: List[str] = []
    for arg in args:
        if isinstance(arg, VariableNameInterface):
            arg_strs.append(f'{arg.variable_name}')
            continue
        arg = string_util.escape_str(string=str(arg))
        arg_strs.append(f'"{arg}"')
    expression += ', '.join(arg_strs)
    expression += ');'
    ap.append_js_expression(expression=expression)



"""This module is for the translation mapping data of the
following document:

Document file: animation_method_chaining.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# AnimationBase class interfaces method chaining':
    '',

    'This page describes the method chaining of the animation interfaces.':
    '',

    '## Each AnimationBase related interface returns its instance':
    '',

    'Each `AnimationBase` class-related interface (such as the `animation_x`\\, `start`\\, `animation_complete`) returns its instance. So that these interfaces can be used with the method chaining, as follows (method chaining of the `animation_x`\\, `animation_complete`\\, and `start` methods):':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(\'Animation is completed!\')\n\n\nap.Stage(\n    stage_width=200, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.animation_x(\n    x=100,\n    duration=1000,\n).animation_complete(on_animation_complete).start()\n\nap.save_overall_html(\n    dest_dir_path=\'./animation_method_chaining_basic_usage_1/\')\n```':  # noqa
    '',

    '<iframe src="static/animation_method_chaining_basic_usage_1/index.html" width="200" height=150></iframe>\n\nThese method chainings are sometimes useful for code simplicity.\n\nIf you want to chain the methods like JavaScript (e.g., `D3.js`), you can use backslashes at the line end, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(\'Animation is completed!\')\n\n\nap.Stage(\n    stage_width=200, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle\\\n    .animation_x(x=100, duration=1000)\\\n    .animation_complete(on_animation_complete)\\\n    .start()\n\nap.save_overall_html(\n    dest_dir_path=\'./animation_method_chaining_basic_usage_2/\')\n```':  # noqa
    '',

    '<iframe src="static/animation_method_chaining_basic_usage_2/index.html" width="200" height=150></iframe>':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: animation_reset.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# animation_reset interface':
    '',

    'This page explains the `animation_reset` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `animation_reset` interface resets all animations and stop.\n\nThis interface exists in the instances that have the animation interfaces (such as the `animation_x`\\, `animation_move`).':  # noqa
    '',

    '## Basic usage':
    '',

    'The following example sets the click event to the rectangle. If you click the rectangle, the x-coordinate animation starts. After 1 second has passed, the `animation_reset` interface resets the x-coordinate animation:':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.Rectangle\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.animation_x(x=500, duration=3000).start()\n    options_: _RectOptions = {\'rectangle\': e.this}\n    timer: ap.Timer = ap.Timer(\n        on_timer, delay=1000, repeat_count=1, options=options_)\n    timer.start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : _RectOptions\n        Optional arguments dictionary.\n    """\n    options[\'rectangle\'].animation_reset()\n\n\nap.Stage(\n    stage_width=600, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(\n    dest_dir_path=\'./animation_reset_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/animation_reset_basic_usage/index.html" width="600" height=150></iframe>':  # noqa
    '',

    '## animation_reset API':
    '',

    '<!-- Docstring: apysc._animation.animation_reset_interface.AnimationResetInterface.animation_reset -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `animation_reset(self) -> None`<hr>\n\n**[Interface summary]** Stop all animations and reset.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n>>> def on_timer(\n...         e: ap.TimerEvent,\n...         options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options[\'rectangle\']\n...     rectangle.animation_reset()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {\'rectangle\': rectangle}\n>>> ap.Timer(on_timer, delay=750, options=options).start()\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: dictionary_length.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Dictionary length interface':
    '',

    'This page explains the `Dictionary` class `length` property interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `length` property returns the length of dictionary keys.':
    '',

    '## Basic usage':
    '',

    'The `length` property interface returns the `Int` value. There is no setter interface.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\ndict_1: ap.Dictionary = ap.Dictionary({\'a\': 10, \'b\': 20})\nassert dict_1.length == 2\nassert isinstance(dict_1.length, ap.Int)\n```':  # noqa
    '',

    '':
    '',

    '## Note for the len function':
    '',

    'The Python built-in `len` function is not supported and raises an exception:':  # noqa
    '',

    '```py\nimport apysc as ap\n\ndict_1: ap.Dictionary = ap.Dictionary({\'a\': 10, \'b\': 20})\nlen(dict_1)\n```':  # noqa
    '',

    '':
    '',

    '```\nException: Dictionary instance can\'t apply len function. Please use length property instead.\n```':  # noqa
    '',

    '':
    '',

    '## length property API':
    '',

    '<!-- Docstring: apysc._type.dictionary.Dictionary.length -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get length of this dictionary values.<hr>\n\n**[Returns]**\n\n- `length`: Int\n  - This dictionary value\'s length.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> dictionary: ap.Dictionary = ap.Dictionary({\'a\': 1, \'b\': 2})\n>>> dictionary.length\nInt(2)\n```':  # noqa
    '',

}



"""Implementation of common cast conversion.

Mainly following interfaces are defined.

- to_int_from_float
    Convert float value to int.
- to_float_from_int
    Convert int value to float.
- to_bool_from_int
    Convert int value to bool.
"""


from typing import Any
from typing import Union

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number_value_interface import NumberValueInterface


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def to_int_from_float(
        *, int_or_float: Union[int, float, NumberValueInterface]) -> Any:
    """
    Convert float value to int.

    Parameters
    ----------
    int_or_float : int or float or Int or Number
        Target float value. If an integer is specified,
        this interface skips the conversion.

    Returns
    -------
    int_val : Int or int
        Converted integer value.
    """
    import apysc as ap
    if isinstance(int_or_float, ap.Number):
        return ap.Int(int_or_float)
    if not isinstance(int_or_float, float):
        return int_or_float
    return int(int_or_float)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def to_float_from_int(
        *, int_or_float: Union[int, float, NumberValueInterface]) -> Any:
    """
    Convert int value to float.

    Parameters
    ----------
    int_or_float : int or float or Int or Number
        A Target integer value. If a float is specified,
        this interface skips the conversion.

    Returns
    -------
    float_val : float or Number
        Converted float value.
    """
    import apysc as ap
    if isinstance(int_or_float, ap.Int):
        return ap.Number(int_or_float)
    if not isinstance(int_or_float, int):
        return int_or_float
    return float(int_or_float)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def to_bool_from_int(*, integer: Union[int, Int]) -> bool:
    """
    Convert int value to bool.

    Parameters
    ----------
    integer : Int or int
        Integer value to convert.

    Returns
    -------
    bool_val : bool
        Converted boolean value.

    Raises
    ------
    ValueError
        If an argument value isn't zero or one.
    """
    from apysc._validation import number_validation
    number_validation.validate_int_is_zero_or_one(integer=integer)
    if integer == 0:
        return False
    return True



"""This module is for the translation mapping data of the
following document:

Document file: sequential_animation.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Sequential animation setting':
    '',

    'This page explains how to animate sequentially.':
    '',

    '## Sequential animation interface calling on the same instance':
    '',

    'If you call each animation interface sequentially, these animations start in order (e.g., when the first animation completion, the second one starts).\n\nThe following example sets the four animations of the coordinates. These animations do not start simultaneously:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\neasing: ap.Easing = ap.Easing.EASE_OUT_QUINT\nrectangle.animation_x(x=100, duration=1000, delay=1000, easing=easing).start()\nrectangle.animation_y(y=100, duration=1000, delay=1000, easing=easing).start()\nrectangle.animation_x(x=50, duration=1000, delay=1000, easing=easing).start()\nrectangle.animation_y(y=50, duration=1000, delay=1000, easing=easing).start()\n\nap.save_overall_html(\n    dest_dir_path=\'sequential_animation_example_1/\')\n```':  # noqa
    '',

    '<iframe src="static/sequential_animation_example_1/index.html" width="200" height="200"></iframe>':  # noqa
    '',

    '## animation_complete handler setting':
    '',

    'Also, you can use the `animation_complete` interface to register a handler for the sequence animation. For the details, please see [animation_complete interface document](animation_complete.md).':  # noqa
    '',

    '## See also':
    '',

    'If you want to animate multiple animations simultaneously, you can use the [`animation_parallel` interface](animation_parallel.md).':  # noqa
    '',

}
